'use strict';

var fs = require('fs');

// Replace '{prop}' in sql with options[prop]
function replace(sql, options) {
  return sql.replace(/{([^}]+)}/g, function(match) {
    var key = match.slice(1, -1);
    if (!options || options[key] === undefined) {
      throw new Error('<Friendly SQL>  Option: ' + key +
                      ' is undefined in sql: ' + sql);
    }
    return options[key];
  });
}

// Parse a file with sql statements and return a function.
function parse(filename) {
  function getId(line) {
    var pattern = /\-\-\s*id:/;
    if (pattern.test(line)) { return line.replace(pattern, '').trim(); }
    return;
  }

  var result = {}, collectedLines = [], currentId;

  fs.readFileSync(filename, 'utf8')
    .replace(/\r/, '')
    .split('\n')
    .forEach(function (line, i) {
      var
        newLine = line.replace(/\-\-.*/, '').trimRight(), // remove comment
        newId = getId(line);

      if (currentId && newLine) { collectedLines.push(newLine); }
      if (newId) {
        result[currentId] = collectedLines.join('\n');
        collectedLines = [];
        if (result[newId]) {
          throw new Error('Parse Error: Duplicate sql id: "' + newId +
                          '" on line ' + i);
        }
        currentId = newId;
      }
    });

  if (currentId) { result[currentId] = collectedLines.join('\n'); }

  function sql(id, options) {
    var rawSql = result[id], replacedSql;

    if (rawSql === undefined) {
      throw new Error('<Friendly SQL>  No such id: ' + id);
    }
    replacedSql = replace(result[id], options);
    if (sql.log) {
      console.log('\x1b[35m<Friendly SQL>\n\x1b[32mid:\x1b[0m', id);
      console.log('\x1b[32mRaw SQL:\x1b[0m\n%s', rawSql);
      if (options) {
        console.log('\x1b[32mOptions:\x1b[0m\n', options);
        console.log('\x1b[32mReplaced SQL:\x1b[0m\n%s', replacedSql);
      }
    }
    return replacedSql;
  }

  return sql;
}

module.exports = {
  parse: parse
};

